"{ Package: 'exept:regression' }"

"{ NameSpace: RegressionTests }"

TestCase subclass:#ComplexTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'tests-Regression'
!


!ComplexTest methodsFor:'tests'!

testAddition
	| c1 c2 |

	self shouldnt: [ c1 := Complex real: 1 imaginary: 2 ] raise: Exception.
	self shouldnt: [ c2 := Complex real: 3 imaginary: 4 ] raise: Exception.

	self should: [ (c1 + c2) = (Complex real: 4 imaginary: 6) ].
!

testCreation
        | c |

        false ifTrue:[
            "cg: I dont think this has to be known outside complex"
            "i.e. should it really be allowed to create complex numbers with new ?"
            self should: [ (c := Complex new) realPart = 0 ].
            self should: [ c imaginaryPart = 0 ].
        ].
        self should: [ (c := Complex real: 1 imaginary: 2) realPart = 1 ].
        self should: [ c imaginaryPart = 2 ]
!

testDivision
	| c1 c2 |

	self shouldnt: [ c1 := Complex real: 2 imaginary: 2 ] raise: Exception.
	self shouldnt: [ c2 := Complex real: 3 imaginary: 6 ] raise: Exception.

	self should: [ (c1 / c1) = (Complex real: 1 imaginary: 0) ].
	self should: [ (c1 / c2) = (Complex real: 2/5 imaginary: (-2/15)) ].
	self should: [ (c2 / c1) = (Complex real: 9/4 imaginary: 3/4) ].

	self should: [ c2 / 3 = (Complex real: 1 imaginary: 2) ].
	self should: [ c1 / 2 = (Complex real: 1 imaginary: 1) ].
!

testEquality
	self should: [ 3 = (Complex real: 3 imaginary: 0) ].
	self should: [ (Complex real: 3 imaginary: 0) = 3 ].

	self should: [ 3.0 = (Complex real: 3 imaginary: 0) ].
	self should: [ (Complex real: 3 imaginary: 0) = 3.0 ].

	self shouldnt: [ 3 = (Complex real: 3 imaginary: 1) ].
	self shouldnt: [ (Complex real: 3 imaginary: 1) = 3 ].

	self shouldnt: [ 3.0 = (Complex real: 3 imaginary: 1) ].
	self shouldnt: [ (Complex real: 3 imaginary: 1) = 3.0 ].

	self should: [ 3 ~= (Complex real: 3 imaginary: 1) ].
	self should: [ (Complex real: 3 imaginary: 1) ~= 3 ].

	self should: [ 3.0 ~= (Complex real: 3 imaginary: 1) ].
	self should: [ (Complex real: 3 imaginary: 1) ~= 3.0 ].
!

testModulus
	| c1 c2 |

	"Test case where |a| < |b| in complex number (a + ib)."

	self shouldnt: [ c1 := Complex real: 2 imaginary: 3 ] raise: Exception.
	self should: [ c1 modulus = (3 * ((1 + ((2 / 3) * (2 / 3))) sqrt)) ].

	"Test case where |a| >= |b| in complex number (a + ib)."

	self shouldnt: [ c2 := Complex real: 4 imaginary: -2 ] raise: Exception.
	self should: [ c2 modulus = (4 * ((1 + ((-2 / 4) * (-2 / 4))) sqrt)) ].
!

testMultiplication
	| c1 c2 |

	self shouldnt: [ c1 := Complex real: 1 imaginary: 2 ] raise: Exception.
	self shouldnt: [ c2 := Complex real: 3 imaginary: 4 ] raise: Exception.

	self should: [ (c1 * c2) = (Complex real: -5 imaginary: 10) ].
	self should: [ (c1 * Complex zero) = Complex zero ].

	self should: [ c1 * 5 = (Complex real: 5 imaginary: 10) ].
	self should: [ c1 * 1.1 = (Complex real: 1.1 imaginary: 2.2) ].
	self should: [ c1 * (2/3) = (Complex real: 2/3 imaginary: 4/3) ].
!

testRaisedTo
	| c |

	self shouldnt: [ c := Complex real: 3 imaginary: 2 ] raise: Exception.
	self should: [ (c raisedTo: 2) = (c * c) ].
	self should: [ (c raisedTo: 3) = (c * c * c) ].
	self should: [ (c raisedTo: 4) = (c * c * c * c) ].
!

testSqrt
        | c w |

        self shouldnt: [ c := Complex real: 0 imaginary: 0 ] raise: Exception.
        self should: [ c sqrt = 0 ].

        self shouldnt: [ c := Complex real: 9 imaginary: 4 ] raise: Exception.
        "cg: that cannot be tested easily, due to rounding errors"
        "original" false ifTrue:[
            self should: [ w := 3 * (((1 + (1 + ((4/9) * (4/9))) sqrt) / 2) sqrt).
                            c sqrt = (Complex real: w imaginary: 4 / (2 * w)) ].
        ] ifFalse:[
            self should: [ |t1 t2 epsilon|
                            w := 3 * (((1 + (1 + ((4/9) * (4/9))) sqrt) / 2) sqrt).
                            t1 := c sqrt.
                            t2 := (Complex real: w imaginary: 4 / (2 * w)).
                            epsilon := 0.0000001.
                            (t1 realPart - t2 realPart) < epsilon
                            and:[ (t1 imaginaryPart - t2 imaginaryPart) < epsilon ]
                         ].
        ].

        self shouldnt: [ c := Complex imaginary: -2 ] raise: Exception.
        "self should: [ c sqrt = (Complex real: 1 imaginary: -1) ].  Should be true, but rounding bites us..."
        self should: [ (c sqrt realPart - 1) abs < 1.0e-10 ].
        self should: [ (c sqrt imaginaryPart + 1) abs < 1.0e-10 ].
!

testSubtraction
	| c1 c2 |

	self shouldnt: [ c1 := Complex real: 1 imaginary: 2 ] raise: Exception.
	self shouldnt: [ c2 := Complex real: 3 imaginary: 4 ] raise: Exception.

	self should: [ (c1 - c2) = (Complex real: -2 imaginary: -2) ].
! !

!ComplexTest class methodsFor:'documentation'!

version
    ^ '$Header$'
! !
